@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright: (c) 2019, Dell EMC
+# Copyright: (c) 2019-2021, Dell EMC
 
 """Collection of provisioning related functions for PowerStore"""
 
@@ -13,11 +13,13 @@ LOG = helpers.get_logger(__name__)
 class Provisioning:
     """Provisioning related functionality for PowerStore."""
     def __init__(self, server_ip, username, password, verify,
-                 application_type, timeout, enable_log=False):
+                 application_type, timeout, enable_log=False, port_no=None):
         """ Initializes Provisioning Class
 
         :param server_ip: The array IP
         :type server_ip: str
+        :param port_no: The port number
+        :type port_no: int
         :param username: array username
         :type username: str
         :param password: array password
@@ -33,10 +35,32 @@ class Provisioning:
         :type enable_log: bool
         """
         global LOG
-        self.server_ip = server_ip
+        if port_no is None:
+            port_no = 443
+        self.server_ip = server_ip + ":" + str(port_no)
         self.client = Client(username, password, verify, application_type,
                              timeout, enable_log=enable_log)
         LOG = helpers.get_logger(__name__, enable_log=enable_log)
+        helpers.set_provisioning_obj(self)
+
+    def get_array_version(self):
+        """Get the software version of this array.
+
+        :return: software version details
+        :rtype: str
+        """
+        LOG.info("Getting the software version of this array")
+        querystring = helpers.prepare_querystring(
+            constants.SELECT_VERSION)
+        LOG.info("Querystring: '%s'" % querystring)
+        sw_versions =  self.client.request(constants.GET,
+                                   constants.GET_SOFTWARE_VERSION.format
+                                   (self.server_ip), payload=None,
+                                   querystring=querystring)
+        version = None
+        if sw_versions and len(sw_versions) >0:
+            version =  sw_versions[0]['release_version']
+        return version
 
     def create_volume(self, name, size, description=None,
                       volume_group_id=None, protection_policy_id=None,
@@ -326,10 +350,13 @@ class Provisioning:
         :rtype: dict
         """
         LOG.info("Getting volume details by ID: '%s'" % volume_id)
+        querystring = constants.SELECT_ALL_VOLUME
+        if helpers.is_foot_hill_or_higher():
+            querystring = constants.FHC_VOLUME_DETAILS_QUERY
         resp = self.client.request(constants.GET,
                                    constants.GET_VOLUME_DETAILS_URL.format
                                    (self.server_ip, volume_id), payload=None,
-                                   querystring=constants.SELECT_ALL_VOLUME)
+                                   querystring=querystring)
 
         hlu_details = self.get_host_volume_mapping(volume_id=volume_id)
 
@@ -432,10 +459,13 @@ class Provisioning:
         :rtype: dict
         """
         LOG.info("Getting host details by ID: '%s'" % host_id)
+        querystring = constants.SELECT_ALL_HOST
+        if helpers.is_foot_hill_or_higher():
+            querystring = constants.FHC_HOST_DETAILS_QUERY
         return self.client.request(constants.GET,
                                    constants.GET_HOST_DETAILS_URL.format(
                                        self.server_ip, host_id), payload=None,
-                                   querystring=constants.SELECT_ALL_HOST)
+                                   querystring=querystring)
 
     def modify_host(self, host_id, name=None, description=None,
                     remove_initiators=None, add_initiators=None,
@@ -1739,3 +1769,75 @@ class Provisioning:
                                    .format(self.server_ip, tree_quota_id))
 
     # FS Quota Methods end
+
+    # Job Methods start
+
+    def get_job_details(self, job_id):
+        """Get details of a Job with its id.
+        :param job_id: The Id of job.
+        :type job_id: str
+        :return: Job details
+        :rtype: dict
+        """
+        LOG.info("Getting job details: '%s'" % job_id)
+        querystring = constants.JOB_DETAILS_QUERY
+        if helpers.is_foot_hill_or_higher():
+            querystring = constants.FHC_JOB_DETAILS_QUERY
+        return self.client.request(
+            constants.GET,
+            constants.GET_JOB_DETAILS_URL.format(self.server_ip, job_id),
+            querystring=querystring
+        )
+
+    # Job Methods end
+
+    # AD methods start
+
+    def get_file_ads(self, filter_dict=None, all_pages=False):
+        """Get a list of active directories.
+
+        :param filter_dict: (optional) Filter detail
+        :type filter_dict: dict
+        :param all_pages: (optional) Indicates whether to return all element
+                          or not
+        :type all_pages: bool
+        :returns: active directories
+        :rtype: list of dict
+        """
+        LOG.info("Getting active directories with filter: '%s' and all_pages: %s"
+                 % (filter_dict, all_pages))
+        querystring = helpers.prepare_querystring(filter_dict)
+        LOG.info("Querystring: '%s'" % querystring)
+        return self.client.request(constants.GET,
+                                   constants.GET_AD_LIST_URL.format
+                                   (self.server_ip), payload=None,
+                                   querystring=querystring,
+                                   all_pages=all_pages)
+
+
+    # AD Methods end
+
+    # LDAP method start
+
+    def get_file_ldaps(self, filter_dict=None, all_pages=False):
+        """Get a list of ldap.
+
+        :param filter_dict: (optional) Filter detail
+        :type filter_dict: dict
+        :param all_pages: (optional) Indicates whether to return all element
+                          or not
+        :type all_pages: bool
+        :returns: ldap
+        :rtype: list of dict
+        """
+        LOG.info("Getting ldap with filter: '%s' and all_pages: %s"
+                 % (filter_dict, all_pages))
+        querystring = helpers.prepare_querystring(filter_dict)
+        LOG.info("Querystring: '%s'" % querystring)
+        return self.client.request(constants.GET,
+                                   constants.GET_LDAP_LIST_URL.format
+                                   (self.server_ip), payload=None,
+                                   querystring=querystring,
+                                   all_pages=all_pages)
+
+    #LDAP method end
